@@ -1,7 +1,7 @@
 // qpwgraph_canvas.cpp
 //
 /****************************************************************************
-   Copyright (C) 2021-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2021-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -40,6 +40,9 @@
 #include <QWheelEvent>
 #include <QKeyEvent>
 
+#include <QGestureEvent>
+#include <QPinchGesture>
+
 #include <algorithm>
 
 #include <cmath>
@@ -65,7 +68,7 @@ static const char *PortAliasesGroup = "/GraphPortAliases";
 qpwgraph_canvas::qpwgraph_canvas ( QWidget *parent )
 	: QGraphicsView(parent), m_state(DragNone), m_item(nullptr),
 		m_connect(nullptr), m_rubberband(nullptr),
-		m_zoom(1.0), m_zoomrange(false),
+		m_zoom(1.0), m_zoomrange(false), m_gesture(false),
 		m_commands(nullptr), m_settings(nullptr), m_patchbay(nullptr),
 		m_patchbay_edit(false), m_patchbay_autopin(true),
 		m_selected_nodes(0), m_repel_overlapping_nodes(false),
@@ -96,6 +99,8 @@ qpwgraph_canvas::qpwgraph_canvas ( QWidget *parent )
 		SIGNAL(editingFinished()),
 		SLOT(editingFinished()));
 
+	QGraphicsView::grabGesture(Qt::PinchGesture);
+
 	m_editor->setEnabled(false);
 	m_editor->hide();
 
@@ -268,8 +273,7 @@ void qpwgraph_canvas::addItem ( qpwgraph_item *item )
 		qpwgraph_node *node = static_cast<qpwgraph_node *> (item);
 		if (node) {
 			m_nodes.append(node);
-			m_node_ids.insert(qpwgraph_node::NodeIdKey(node), node);
-			m_node_keys.insert(qpwgraph_node::NodeNameKey(node), node);
+			addNodeKeys(node);
 			if (restoreNode(node))
 				emit updated(node);
 			else
@@ -300,8 +304,7 @@ void qpwgraph_canvas::removeItem ( qpwgraph_item *item )
 		if (node && saveNode(node)) {
 			emit removed(node);
 			node->removePorts();
-			m_node_keys.remove(qpwgraph_node::NodeNameKey(node));
-			m_node_ids.remove(qpwgraph_node::NodeIdKey(node));
+			removeNodeKeys(node);
 			m_nodes.removeAll(node);
 		}
 	}
@@ -479,8 +482,8 @@ void qpwgraph_canvas::clearNodes ( uint node_type )
 
 	foreach (qpwgraph_node *node, m_nodes) {
 		if (node->nodeType() == node_type) {
-			m_node_keys.remove(qpwgraph_node::NodeNameKey(node));
-			m_node_ids.remove(qpwgraph_node::NodeIdKey(node));
+			m_node_names.remove(qpwgraph_node::NodeNameKey(node), node);
+			m_node_ids.remove(qpwgraph_node::NodeIdKey(node), node);
 			m_nodes.removeAll(node);
 			nodes.append(node);
 		}
@@ -494,8 +497,14 @@ void qpwgraph_canvas::clearNodes ( uint node_type )
 qpwgraph_node *qpwgraph_canvas::findNode (
 	uint id, qpwgraph_item::Mode mode, uint type ) const
 {
-	return static_cast<qpwgraph_node *> (
-		m_node_ids.value(qpwgraph_node::IdKey(id, mode, type), nullptr));
+	return m_node_ids.value(qpwgraph_item::IdKey(id, mode, type), nullptr);
+}
+
+
+qpwgraph_node *qpwgraph_canvas::findNode (
+	const QString& name, qpwgraph_item::Mode mode, uint type ) const
+{
+	return m_node_names.value(qpwgraph_node::NodeNameKey(name, mode, type), nullptr);
 }
 
 
@@ -507,10 +516,11 @@ bool qpwgraph_canvas::isBusy (void) const
 }
 
 
-QList<qpwgraph_node *> qpwgraph_canvas::findNodes (
-	const QString& name, qpwgraph_item::Mode mode, uint type ) const
+void qpwgraph_canvas::releaseNode ( qpwgraph_node *node )
 {
-	return m_node_keys.values(qpwgraph_node::NodeNameKey(name, mode, type));
+	removeNodeKeys(node);
+
+	node->setMarked(false);
 }
 
 
@@ -518,6 +528,12 @@ QList<qpwgraph_node *> qpwgraph_canvas::findNodes (
 void qpwgraph_canvas::emitConnectPorts (
 	qpwgraph_port *port1, qpwgraph_port *port2, bool is_connect )
 {
+	if (m_patchbay_edit && !m_patchbay_autopin && is_connect) {
+		qpwgraph_connect *connect = port1->findConnect(port2);
+		if (connect)
+			connect->setDimmed(true);
+	}
+
 	if (m_patchbay && (m_patchbay_autopin || !is_connect))
 		m_patchbay->connectPorts(port1, port2, is_connect);
 
@@ -589,24 +605,39 @@ void qpwgraph_canvas::connectPorts (
 // Mouse event handlers.
 void qpwgraph_canvas::mousePressEvent ( QMouseEvent *event )
 {
+	if (m_gesture)
+		return;
+
 	m_state = DragNone;
 	m_item = nullptr;
 	m_pos = QGraphicsView::mapToScene(event->pos());
 
-	qpwgraph_item *item = itemAt(m_pos);
-	if (item && item->type() >= QGraphicsItem::UserType)
-		m_item = static_cast<qpwgraph_item *> (item);
-
-	if (event->button() == Qt::LeftButton ||
-		event->button() == Qt::MiddleButton)
+	if (event->button() == Qt::LeftButton) {
+		m_item = itemAt(m_pos);
 		m_state = DragStart;
+	}
 
-	if (m_state == DragStart && m_item == nullptr
+	if (m_item == nullptr
 		&& (((event->button() == Qt::LeftButton)
 		  && (event->modifiers() & Qt::ControlModifier))
 		  || (event->button() == Qt::MiddleButton))
 		&& m_scene->selectedItems().isEmpty()) {
-		QGraphicsView::setCursor(Qt::ClosedHandCursor);
+	#if 1//NEW_DRAG_SCROLL_MODE
+		// HACK: When about to drag-scroll,
+		// always fake a left-button press...
+		QGraphicsView::setDragMode(ScrollHandDrag);
+		QMouseEvent event2(event->type(),
+		#if QT_VERSION >= QT_VERSION_CHECK(6, 1, 0)
+			event->position(), event->globalPosition(),
+		#else
+			event->localPos(), event->globalPos(),
+		#endif
+			Qt::LeftButton, Qt::LeftButton,
+			event->modifiers() | Qt::ControlModifier);
+		QGraphicsView::mousePressEvent(&event2);
+	#else
+		QGraphicsView::setCursor(Qt::ClosedHandCursor)
+	#endif
 		m_state = DragScroll;
 	}
 }
@@ -614,6 +645,9 @@ void qpwgraph_canvas::mousePressEvent ( QMouseEvent *event )
 
 void qpwgraph_canvas::mouseMoveEvent ( QMouseEvent *event )
 {
+	if (m_gesture)
+		return;
+
 	int nchanged = 0;
 
 	QPointF pos = QGraphicsView::mapToScene(event->pos());
@@ -654,10 +688,8 @@ void qpwgraph_canvas::mouseMoveEvent ( QMouseEvent *event )
 						++nchanged;
 					}
 					// Original node position (for move command)...
-					QPointF pos1 = m_pos;
-					pos1.setX(4.0 * ::round(0.25 * pos1.x()));
-					pos1.setY(4.0 * ::round(0.25 * pos1.y()));
-					m_pos1 = pos1;
+					m_pos1 = m_pos;
+					snapPos(m_pos1);
 				}
 				else m_item = nullptr;
 			}
@@ -667,19 +699,12 @@ void qpwgraph_canvas::mouseMoveEvent ( QMouseEvent *event )
 				m_rubberband = new QRubberBand(QRubberBand::Rectangle, this);
 			}
 			// Set allowed auto-scroll margins/limits...
-			const QRect& rect = QGraphicsView::rect();
-			const qreal mx = 0.5f * rect.width();
-			const qreal my = 0.5f * rect.height();
-			m_rect1 = m_scene->itemsBoundingRect()
-				.marginsAdded(QMarginsF(mx, my, mx, my));
+			boundingRect(true);
 		}
 		break;
 	case DragMove:
 		// Allow auto-scroll only if within allowed margins/limits...
-		if (!m_rect1.contains(pos)) {
-			pos.setX(qBound(m_rect1.left(), pos.x(), m_rect1.right()));
-			pos.setY(qBound(m_rect1.top(),  pos.y(), m_rect1.bottom()));
-		}
+		boundingPos(pos);
 		QGraphicsView::ensureVisible(QRectF(pos, QSizeF(2, 2)), 8, 8);
 		// Move new connection line...
 		if (m_connect)
@@ -731,8 +756,7 @@ void qpwgraph_canvas::mouseMoveEvent ( QMouseEvent *event )
 		}
 		// Move current selected nodes...
 		if (m_item && m_item->type() == qpwgraph_node::Type) {
-			pos.setX(4.0 * ::round(0.25 * pos.x()));
-			pos.setY(4.0 * ::round(0.25 * pos.y()));
+			snapPos(pos);
 			const QPointF delta = (pos - m_pos);
 			foreach (QGraphicsItem *item, m_scene->selectedItems()) {
 				if (item->type() == qpwgraph_node::Type) {
@@ -762,12 +786,16 @@ void qpwgraph_canvas::mouseMoveEvent ( QMouseEvent *event )
 		}
 		break;
 	case DragScroll: {
+	#if 1//NEW_DRAG_SCROLL_MODE
+		QGraphicsView::mouseMoveEvent(event);
+	#else
 		QScrollBar *hbar = QGraphicsView::horizontalScrollBar();
 		QScrollBar *vbar = QGraphicsView::verticalScrollBar();
 		const QPoint delta = (pos - m_pos).toPoint();
 		hbar->setValue(hbar->value() - delta.x());
 		vbar->setValue(vbar->value() - delta.y());
 		m_pos = pos;
+	#endif
 		break;
 	}
 	default:
@@ -781,6 +809,9 @@ void qpwgraph_canvas::mouseMoveEvent ( QMouseEvent *event )
 
 void qpwgraph_canvas::mouseReleaseEvent ( QMouseEvent *event )
 {
+	if (m_gesture)
+		return;
+
 	int nchanged = 0;
 
 	switch (m_state) {
@@ -878,6 +909,12 @@ void qpwgraph_canvas::mouseReleaseEvent ( QMouseEvent *event )
 		break;
 	}
 
+#if 1//NEW_DRAG_SCROLL_MODE
+	if (QGraphicsView::dragMode() == ScrollHandDrag) {
+		QGraphicsView::mouseReleaseEvent(event);
+		QGraphicsView::setDragMode(NoDrag);
+	}
+#endif
 	m_state = DragNone;
 	m_item = nullptr;
 
@@ -1251,11 +1288,7 @@ bool qpwgraph_canvas::restoreNode ( qpwgraph_node *node )
 	if (m_settings == nullptr || node == nullptr)
 		return false;
 
-	// Assume node name-keys have been added before this...
-	//
-	const qpwgraph_node::NodeNameKey name_key(node);
-	const int n = m_node_keys.values(name_key).count();
-	const QString& node_key = nodeKey(node, n);
+	const QString& node_key = nodeKey(node);
 
 	m_settings->beginGroup(NodeAliasesGroup);
 	const QString& node_title
@@ -1266,13 +1299,14 @@ bool qpwgraph_canvas::restoreNode ( qpwgraph_node *node )
 		node->setNodeTitle(node_title);
 
 	m_settings->beginGroup(NodePosGroup);
-	const QPointF& node_pos
-		= m_settings->value('/' + node_key).toPointF();
+	QPointF node_pos = m_settings->value('/' + node_key).toPointF();
 	m_settings->endGroup();
 
 	if (node_pos.isNull())
 		return false;
 
+	boundingPos(node_pos);
+
 	node->setPos(node_pos);
 	return true;
 }
@@ -1283,14 +1317,10 @@ bool qpwgraph_canvas::saveNode ( qpwgraph_node *node ) const
 	if (m_settings == nullptr || node == nullptr)
 		return false;
 
-	// Assume node name-keys are to be removed after this...
-	//
-	const qpwgraph_node::NodeNameKey name_key(node);
-	const int n = m_node_keys.values(name_key).count();
-	const QString& node_key = nodeKey(node, n);
+	const QString& node_key = nodeKey(node);
 
 	m_settings->beginGroup(NodeAliasesGroup);
-	if (node->nodeName() != node->nodeTitle()) {
+	if (node->nodeNameLabel() != node->nodeTitle()) {
 		m_settings->setValue('/' + node_key, node->nodeTitle());
 	} else {
 		m_settings->remove('/' + node_key);
@@ -1333,7 +1363,7 @@ bool qpwgraph_canvas::savePort ( qpwgraph_port *port ) const
 	const QString& port_key = portKey(port);
 
 	m_settings->beginGroup(PortAliasesGroup);
-	if (port->portName() != port->portTitle())
+	if (port->portNameLabel() != port->portTitle())
 		m_settings->setValue('/' + port_key, port->portTitle());
 	else
 		m_settings->remove('/' + port_key);
@@ -1348,6 +1378,10 @@ bool qpwgraph_canvas::restoreState (void)
 	if (m_settings == nullptr)
 		return false;
 
+#ifdef CONFIG_CLEANUP_NODE_NAMES
+	cleanupNodeNames(NodePosGroup);
+	cleanupNodeNames(NodeAliasesGroup);
+#endif
 	m_settings->beginGroup(ColorsGroup);
 	const QRegularExpression rx("^0x");
 	QStringListIterator key(m_settings->childKeys());
@@ -1364,13 +1398,12 @@ bool qpwgraph_canvas::restoreState (void)
 	m_settings->endGroup();
 
 	m_settings->beginGroup(CanvasGroup);
-	m_settings->setValue(CanvasRectKey, QGraphicsView::sceneRect());
 	const QRectF& rect = m_settings->value(CanvasRectKey).toRectF();
 	const qreal zoom = m_settings->value(CanvasZoomKey, 1.0).toReal();
 	m_settings->endGroup();
 
 	if (rect.isValid())
-		QGraphicsView::setSceneRect(rect);
+		m_rect1 = rect; // QGraphicsView::setSceneRect(rect);
 
 	setZoom(zoom);
 
@@ -1390,22 +1423,17 @@ bool qpwgraph_canvas::saveState (void) const
 		if (item->type() == qpwgraph_node::Type) {
 			qpwgraph_node *node = static_cast<qpwgraph_node *> (item);
 			if (node && !nodes.contains(node)) {
-				int n = 0;
-				const QList<qpwgraph_node *>& nodes2
-					= m_node_keys.values(qpwgraph_node::NodeNameKey(node));
-				foreach (qpwgraph_node *node2, nodes2) {
-					const QString& node2_key = nodeKey(node2, ++n);
-					m_settings->beginGroup(NodePosGroup);
-					m_settings->setValue('/' + node2_key, node2->pos());
-					m_settings->endGroup();
-					m_settings->beginGroup(NodeAliasesGroup);
-					if (node2->nodeName() != node2->nodeTitle())
-						m_settings->setValue('/' + node2_key, node2->nodeTitle());
-					else
-						m_settings->remove('/' + node2_key);
-					m_settings->endGroup();
-					nodes.append(node2);
-				}
+				const QString& node_key = nodeKey(node);
+				m_settings->beginGroup(NodePosGroup);
+				m_settings->setValue('/' + node_key, node->pos());
+				m_settings->endGroup();
+				m_settings->beginGroup(NodeAliasesGroup);
+				if (node->nodeName() != node->nodeTitle())
+					m_settings->setValue('/' + node_key, node->nodeTitle());
+				else
+					m_settings->remove('/' + node_key);
+				m_settings->endGroup();
+				nodes.append(node);
 			}
 		}
 		else
@@ -1425,7 +1453,7 @@ bool qpwgraph_canvas::saveState (void) const
 
 	m_settings->beginGroup(CanvasGroup);
 	m_settings->setValue(CanvasZoomKey, zoom());
-	m_settings->setValue(CanvasRectKey, QGraphicsView::sceneRect());
+	m_settings->setValue(CanvasRectKey, m_rect1);
 	m_settings->endGroup();
 
 	m_settings->beginGroup(ColorsGroup);
@@ -1445,13 +1473,9 @@ bool qpwgraph_canvas::saveState (void) const
 
 
 // Graph node/port key helpers.
-QString qpwgraph_canvas::nodeKey ( qpwgraph_node *node, int n ) const
+QString qpwgraph_canvas::nodeKey ( qpwgraph_node *node ) const
 {
 	QString node_key = node->nodeName();
-	if (n > 1) {
-		node_key += '-';
-		node_key += QString::number(n - 1);
-	}
 
 	switch (node->nodeMode()) {
 	case qpwgraph_item::Input:
@@ -1468,7 +1492,7 @@ QString qpwgraph_canvas::nodeKey ( qpwgraph_node *node, int n ) const
 }
 
 
-QString qpwgraph_canvas::portKey ( qpwgraph_port *port, int n ) const
+QString qpwgraph_canvas::portKey ( qpwgraph_port *port ) const
 {
 	QString port_key;
 
@@ -1479,10 +1503,6 @@ QString qpwgraph_canvas::portKey ( qpwgraph_port *port, int n ) const
 	port_key += node->nodeName();
 	port_key += ':';
 	port_key += port->portName();
-	if (n > 1) {
-		port_key += '-';
-		port_key += QString::number(n - 1);
-	}
 
 	switch (port->portMode()) {
 	case qpwgraph_item::Input:
@@ -1499,6 +1519,20 @@ QString qpwgraph_canvas::portKey ( qpwgraph_port *port, int n ) const
 }
 
 
+void qpwgraph_canvas::addNodeKeys ( qpwgraph_node *node )
+{
+	m_node_ids.insert(qpwgraph_node::NodeIdKey(node), node);
+	m_node_names.insert(qpwgraph_node::NodeNameKey(node), node);
+}
+
+
+void qpwgraph_canvas::removeNodeKeys ( qpwgraph_node *node )
+{
+	m_node_names.remove(qpwgraph_node::NodeNameKey(node), node);
+	m_node_ids.remove(qpwgraph_node::NodeIdKey(node), node);
+}
+
+
 // Graph port colors management.
 void qpwgraph_canvas::setPortTypeColor (
 	uint port_type, const QColor& port_color )
@@ -1692,4 +1726,156 @@ void qpwgraph_canvas::repelOverlappingNodesAll (
 }
 
 
+// Gesture event handlers.
+//
+bool qpwgraph_canvas::event ( QEvent *event )
+{
+	if (event->type() == QEvent::Gesture)
+		return gestureEvent(static_cast<QGestureEvent *> (event));
+	else
+		return QGraphicsView::event(event);
+}
+
+
+bool qpwgraph_canvas::gestureEvent ( QGestureEvent *event )
+{
+	if (QGesture *pinch = event->gesture(Qt::PinchGesture))
+		pinchGesture(static_cast<QPinchGesture *> (pinch));
+
+	return true;
+}
+
+
+void qpwgraph_canvas::pinchGesture ( QPinchGesture *pinch )
+{
+	switch (pinch->state()) {
+	case Qt::GestureStarted: {
+		const qreal scale_factor = zoom();
+		pinch->setScaleFactor(scale_factor);
+		pinch->setLastScaleFactor(scale_factor);
+		pinch->setTotalScaleFactor(scale_factor);
+		m_gesture = true;
+		break;
+	}
+	case Qt::GestureFinished:
+		m_gesture = false;
+		// Fall thru...
+	case Qt::GestureUpdated:
+		if (pinch->changeFlags() & QPinchGesture::ScaleFactorChanged)
+			setZoom(pinch->totalScaleFactor());
+		// Fall thru...
+	default:
+		break;
+	}
+}
+
+
+// Bounding margins/limits...
+//
+const QRectF& qpwgraph_canvas::boundingRect ( bool reset )
+{
+	if (!m_rect1.isValid() || reset) {
+		const QRect& rect = QGraphicsView::rect();
+		const qreal mx = 0.33 * rect.width();
+		const qreal my = 0.33 * rect.height();
+		m_rect1 = m_scene->itemsBoundingRect()
+			.marginsAdded(QMarginsF(mx, my, mx, my));
+	}
+
+	return m_rect1;
+}
+
+
+void qpwgraph_canvas::boundingPos ( QPointF& pos )
+{
+	const QRectF& rect = boundingRect();
+	if (!rect.contains(pos)) {
+		pos.setX(qBound(rect.left(), pos.x(), rect.right()));
+		pos.setY(qBound(rect.top(),  pos.y(), rect.bottom()));
+	}
+}
+
+
+// Snap into position helpers.
+//
+void qpwgraph_canvas::snapPos ( QPointF& pos ) const
+{
+	pos.setX(4.0 * ::round(0.25 * pos.x()));
+	pos.setY(4.0 * ::round(0.25 * pos.y()));
+}
+
+
+QPointF qpwgraph_canvas::snapPos ( qreal x, qreal y ) const
+{
+	QPointF pos(x, y);
+	snapPos(pos);
+	return pos;
+}
+
+
+#ifdef CONFIG_CLEANUP_NODE_NAMES
+
+void qpwgraph_canvas::cleanupNodeNames ( const char *group )
+{
+	bool cleanup = false;
+
+	m_settings->beginGroup("/CleanupNodeNames");
+	cleanup = m_settings->value(group).toBool();
+	if (!cleanup)
+		m_settings->setValue(group, true);
+	m_settings->endGroup();
+
+	if (cleanup)
+		return;
+
+	m_settings->beginGroup(group);
+	const QRegularExpression rx("\\-([0-9]+).*$");
+	QHash<QString, QVariant> keys;
+	QStringListIterator iter(m_settings->childKeys());
+	while (iter.hasNext()) {
+		const QString& key = iter.next();
+		const QVariant& value = m_settings->value(key);
+		QString key2 = key;
+		if (cleanupNodeName(key2)) {
+			int n = 0;
+			if (keys.find(key2) != keys.end()) {
+				const QRegularExpressionMatch mx = rx.match(key2);
+				if (mx.hasMatch()) {
+					n = mx.captured(1).toInt();
+					key2.remove(rx);
+				}
+				QString key3;
+				do { key3 = key2 + '-' + QString::number(++n); }
+				while (keys.find(key3) != keys.end());
+				key2 = key3;
+			}
+			if (n == 0) {
+				keys.insert(key2, value);
+				m_settings->setValue(key2, value);
+			}
+			m_settings->remove(key);
+		} else {
+			keys.insert(key, value);
+		}
+	}
+	m_settings->endGroup();
+}
+
+
+bool qpwgraph_canvas::cleanupNodeName ( QString& name )
+{
+	const QRegularExpression rx("^.+( \\[.+\\])[^ ]*$");
+	const QRegularExpressionMatch& mx = rx.match(name);
+	if (mx.hasMatch()) {
+		name.remove(mx.captured(1));
+		return true;
+	} else {
+		return false;
+	}
+}
+
+#endif//CONFIG_CLEANUP_NODE_NAMES
+
+
+
 // end of qpwgraph_canvas.cpp
