@@ -1,7 +1,7 @@
 // qpwgraph_canvas.h
 //
 /****************************************************************************
-   Copyright (C) 2021-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2021-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -43,9 +43,16 @@ class QMouseEvent;
 class QWheelEvent;
 class QKeyEvent;
 
+class QGestureEvent;
+class QPinchGesture;
+
 class qpwgraph_patchbay;
 
 
+// Define if cleanup of legacy node names is needed (v0.5.0)...
+#define CONFIG_CLEANUP_NODE_NAMES 1
+
+
 //----------------------------------------------------------------------------
 // qpwgraph_canvas -- Canvas graphics scene/view.
 
@@ -114,9 +121,11 @@ public:
 	// Special node finders.
 	qpwgraph_node *findNode(
 		uint id, qpwgraph_item::Mode mode, uint type = 0) const;
-	QList<qpwgraph_node *> findNodes(
+	qpwgraph_node *findNode(
 		const QString& name, qpwgraph_item::Mode mode, uint type = 0) const;
 
+	void releaseNode(qpwgraph_node *node);
+
 	// Whether it's in the middle of something...
 	bool isBusy() const;
 
@@ -156,6 +165,13 @@ public:
 	// Clear all state.
 	void clear();
 
+	// Snap into position helper.
+	QPointF snapPos(qreal x, qreal y) const;
+
+#ifdef CONFIG_CLEANUP_NODE_NAMES
+	static bool cleanupNodeName(QString& name);
+#endif
+
 signals:
 
 	// Node factory notifications.
@@ -227,9 +243,17 @@ protected:
 	// Keyboard event handler.
 	void keyPressEvent(QKeyEvent *event);
 
+	// Gesture event handlers.
+	bool event(QEvent *event);
+	bool gestureEvent(QGestureEvent *event);
+	void pinchGesture(QPinchGesture *pinch);
+
 	// Graph node/port key helpers.
-	QString nodeKey(qpwgraph_node *node, int n = 0) const;
-	QString portKey(qpwgraph_port *port, int n = 0) const;
+	QString nodeKey(qpwgraph_node *node) const;
+	QString portKey(qpwgraph_port *port) const;
+
+	void addNodeKeys(qpwgraph_node *node);
+	void removeNodeKeys(qpwgraph_node *node);
 
 	// Zoom in rectangle range.
 	void zoomFitRange(const QRectF& range_rect);
@@ -244,6 +268,17 @@ protected:
 	// Renaming editor position and size updater.
 	void updateEditorGeometry();
 
+	// Bounding margins/limits...
+	const QRectF& boundingRect(bool reset = false);
+	void boundingPos(QPointF& pos);
+
+	// Snap into position helper.
+	void snapPos(QPointF& pos) const;
+
+#ifdef CONFIG_CLEANUP_NODE_NAMES
+	void cleanupNodeNames(const char *group);
+#endif
+
 private:
 
 	// Mouse pointer dragging states.
@@ -258,10 +293,11 @@ private:
 	QRubberBand      *m_rubberband;
 	qreal             m_zoom;
 	bool              m_zoomrange;
+	bool              m_gesture;
 
-	qpwgraph_node::IdKeys   m_node_ids;
-	qpwgraph_node::NodeKeys m_node_keys;
-	QList<qpwgraph_node *>  m_nodes;
+	qpwgraph_node::NodeIds   m_node_ids;
+	qpwgraph_node::NodeNames m_node_names;
+	QList<qpwgraph_node *>   m_nodes;
 
 	QUndoStack *m_commands;
 	QSettings  *m_settings;
