@@ -1,7 +1,7 @@
 // qpwgraph_config.h
 //
 /****************************************************************************
-   Copyright (C) 2021-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2021-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -101,6 +101,9 @@ public:
 	void setSystemTrayEnabled(bool enabled);
 	bool isSystemTrayEnabled() const;
 
+	void setSessionStartMinimized(bool start_minimized);
+	bool isSessionStartMinimized() const;
+
 	// Graph main-widget state methods.
 	bool restoreState(QMainWindow *widget);
 	bool saveState(QMainWindow *widget) const;
